{# needs sdoc_entity, see README.txt #}

{# multiline fields #}
{# goes through all UNRESERVED fields #}
{%- if sdoc_entity.has_meta %}
  {% for meta_field in sdoc_entity.enumerate_meta_fields(skip_single_lines=True) if view_object.current_view.includes_field(sdoc_entity.node_type, meta_field[0]) %}
    <sdoc-node-field-label>{{ meta_field[0] }}:</sdoc-node-field-label>
    <sdoc-node-field
      data-field-label="{{ meta_field[0] }}"
    >
      {%- with field_content = view_object.render_node_field(meta_field[1]) %}
        {%- include "components/field/index.jinja" -%}
      {%- endwith -%}
    </sdoc-node-field>
  {%- endfor %}
{%- endif %}
