<sdoc-form-row
  {% block row_form_attributes %}
    {% assert 0, "Must not reach here!" %}
  {% endblock row_form_attributes %}
>
  <sdoc-form-row-aside>
    {% block row_left %}
      {% assert 0, "Must not reach here!" %}
    {% endblock row_left %}
  </sdoc-form-row-aside>

  <sdoc-form-row-main>

    {% block row_content %}
      {% assert 0, "Must not reach here!" %}
    {% endblock row_content %}

  </sdoc-form-row-main>

  <sdoc-form-row-aside>
    {% block row_right %}
      {% assert 0, "Must not reach here!" %}
    {% endblock row_right %}
  </sdoc-form-row-aside>
</sdoc-form-row>
