{% extends "components/node/readonly.jinja" %}
{% assert node is defined %}
{%- set sdoc_entity = node -%}
{% block sdoc_entity %}
{% include "components/node_content/index.jinja" %}
{% endblock sdoc_entity %}
