<turbo-stream action="update" target="confirm">

  {#
    Required Variable:
    confirm_href

    Optional Variables:
    confirm_name - confirmation button name
    confirm_title - modal title
    confirm_message - confirmation message
  #}

  <template>
  {% with
          confirm_title = default,
          confirm_message = default,
          confirm_name = "Delete requirement",
          confirm_href = "/actions/document/delete_requirement/"~requirement_mid~"?confirmed=1"
  %}
    {% include "components/confirm/index.jinja" %}
  {% endwith %}
  </template>
</turbo-stream>
