<turbo-stream action="update" target="frame_project_tree">
  <template>
    {% include "screens/project_index/project_tree.jinja" %}
  </template>
</turbo-stream>

<turbo-stream action="update" target="modal">
  <template></template>
</turbo-stream>
