{%- with
  resizable_bar_content = "screens/traceability_matrix/project_tree_flat_anchor_list.jinja",
  resizable_bar_name = "tree"
-%}
  {% include "components/resizable_bar/index.jinja" %}
{%- endwith -%}
