<svg class="svg_icon" width="16px" height="16px" viewBox="0 0 16 16" version="1.1" xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink">
  <path d="M8,10 C8,10 8,2 8,2"></path>
  <polyline points="4 5 8 2 12 5"></polyline>
  <line x1="6" y1="14" x2="10" y2="14"></line>
</svg>
