{#
  In this template:
  sdoc_entity = text_node
#}

<sdoc-text>
  {% include "components/node_field/section_title/index.jinja" %}
  {% include "components/node_field/uid_standalone/index.jinja" %}
  {% include "components/node_field/text/index.jinja" %}
</sdoc-text>
