{% assert folder is defined %}

<tr class="project_coverage-folder">
  <td colspan="99" style="padding-left:{{ folder.level }}0px">
    <div class="project_coverage-folder-title">
      {%- include "_res/svg_ico16_folder.jinja" -%}
      {{ folder.folder_name }}/
      {# {%- include "_res/svg_ico16_folder_collapse.jinja" -%} #}
    </div>

  </td>
</tr>

{% for folder_ in folder.subfolder_trees %}
  {% if folder_.has_content() %}
  {% with folder = folder_ %}
    {% include "screens/source_file_coverage/folder.jinja" %}
  {% endwith %}
  {% endif %}
{% endfor %}

{% for file_ in folder.files %}
  {% with file = file_ %}
    {% include "screens/source_file_coverage/file.jinja" %}
  {% endwith %}
{% endfor %}
