{# comments #}
{# needs sdoc_entity, see README.txt #}

{% if view_object.current_view.includes_field(sdoc_entity.node_type, "COMMENT") %}
  {%- for comment_field_ in sdoc_entity.get_comment_fields() %}
    <sdoc-node-field-label>{{ sdoc_entity.get_field_human_title("COMMENT") }}:</sdoc-node-field-label>
    <sdoc-node-field
      data-field-label="comment"
    >
      {%- with field_content = view_object.render_node_field(comment_field_) -%}
        {%- include "components/field/index.jinja" -%}
      {%- endwith -%}
    </sdoc-node-field>
  {%- endfor %}
{%- endif %}
