{% extends "components/modal/form.jinja" %}
  {#
    Can be defined by calling the Confirm:
    confirm_title
    confirm_message
  #}
{% block modal__context %}confirm{% endblock modal__context %}
{% block modal_form__header %}
  {{ confirm_title|default("Are you sure?") }}
{% endblock modal_form__header %}
{% block modal_form__content %}
<sdoc-modal-message data-testid="confirm-message">
  {{ confirm_message|default(
    "This is an unrecoverable action."
  ) }}

  {# {% set errors = ['test error 1', 'test error 2'] %} #}
  {% if errors|length > 0 %}
    <div class="error">
    <p>This node cannot be deleted:</p>
    <ul>
  {% for error_ in errors %}
    <li>{{ error_ }}</li>
  {% endfor %}
    </ul>
    </div>
  {% endif %}

</sdoc-modal-message>
{% endblock modal_form__content %}
{% block modal_form__footer_submit %}
  {#
    Must be defined by calling the Confirm:
    confirm_href

    Can be defined by calling the Confirm:
    confirm_name
    confirm_turbo_method
    confirm_action_type
  #}
  {% include "components/button/confirm.jinja" %}
{% endblock modal_form__footer_submit %}
