<meta name="strictdoc-project-hash" content="{{ view_object.project_config.get_project_hash() }}">
<meta name="strictdoc-search-index-timestamp" content="{{ view_object.traceability_index.search_index_timestamp.timestamp() }}">
<meta name="strictdoc-search-index-path" content="{{ view_object.render_static_url('static_html_search_index.js') }}">

<script src="{{ view_object.render_static_url('static_html_search.js') }}" defer></script>
