{# toc_position are expected from the 'base' template level  #}
{%- with
  resizable_bar_content = "screens/document/_shared/frame_toc.jinja",
  resizable_bar_name = "toc",
  resizable_bar_position = toc_position|default('left', true)
-%}
  {% include "components/resizable_bar/index.jinja" %}
{%- endwith -%}
