<div class="header">

  {# 0: shared by all screens #}
  {% include "_res/svg_ico16_project.jinja.html" %}
  <div
    class="header__project_name"
    title="{{ view_object.project_config.project_title }}"
  >{{ view_object.project_config.project_title }}
  </div>

  {# 1: blocks in header: #}
  {%- if header__items is defined -%}
    {% for item in header__items %}
      {% include "_res/svg__separator.jinja.html" %}
      {% include item %}
    {% endfor %}
  {%- endif -%}

  {# 2: pagetype #}
  {%- if header__pagetype is defined -%}
    {% include "_res/svg__separator.jinja.html" %}
    {% include "components/header/header_pagetype.jinja" %}
  {%- endif -%}

  {# 3: last block, pushed to the right #}
  {%- if header__last is defined -%}
    {% include header__last %}
  {%- endif -%}

</div>
