<svg class="svg_icon" width="16px" height="16px" viewBox="0 0 16 16" version="1.1" xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink">
  <polyline points="4 11 8 14 12 11"></polyline>
  <line x1="6" y1="2" x2="10" y2="2"></line>
  <line x1="8" y1="14" x2="8" y2="6"></line>
</svg>
