{%- assert not view_object.standalone -%}
<div class="viewtype">
  <div
    id="viewtype_handler"
    class="viewtype__handler"
    aria-expanded="false"
    aria-controls="viewtype_menu"
  >
    <span>{{ view_object.get_page_title() }}</span>
    {% include "_res/svg_ico16_expand.jinja.html" %}
  </div>
  <menu
    id="viewtype_menu"
    class="viewtype__menu"
    aria-hidden="true"
    aria-labelledby="viewtype_handler"
  >
    <li class="viewtype__menu_header">VIEWS</li>
    <li>
      <a
        data-viewtype_link="document"
        class="viewtype__menu_item"
        title="Go to Document view"
        href="{{ view_object.render_document_link(view_object.document, none, "DOCUMENT") }}">
      Document</a>
    </li>
    {%- if view_object.project_config.is_activated_table_screen() -%}
    <li>
      <a
        data-viewtype_link="table"
        class="viewtype__menu_item"
        title="Go to Table view"
        href="{{ view_object.render_document_link(view_object.document, none, "TABLE") }}">
      Table</a>
    </li>
    {%- endif -%}
    {%- if view_object.project_config.is_activated_trace_screen() -%}
    <li>
      <a
        data-viewtype_link="traceability"
        class="viewtype__menu_item"
        title="Go to Traceability view"
        href="{{ view_object.render_document_link(view_object.document, none, "TRACE") }}">
      Traceability</a>
    </li>
    {%- endif -%}
    {%- if view_object.project_config.is_activated_deep_trace_screen() -%}
    <li>
      <a
        data-viewtype_link="deep_traceability"
        class="viewtype__menu_item"
        title="Go to Deep Traceability view"
        href="{{ view_object.render_document_link(view_object.document, none, "DEEPTRACE") }}">
      Deep Traceability</a>
    </li>
    {%- endif -%}
    {%- if view_object.project_config.is_activated_standalone_document() -%}
    <li>
      <a
        data-viewtype_link="standalone_document"
        class="viewtype__menu_item"
        title="Go to Standalone Document view"
        href="{{ view_object.render_standalone_document_link(view_object.document) }}">
      Standalone</a>
    </li>
    {%- endif -%}
    {%- if view_object.project_config.is_activated_html2pdf() -%}
    <li>
      <a
        data-viewtype_link="html2pdf"
        class="viewtype__menu_item"
        title="Go to PDF view"
        href="{{ view_object.render_document_link(view_object.document, none, "PDF") }}">
      PDF</a>
    </li>
    {%- endif -%}
  </menu>
</div>
