<button
  uid="{{ uid }}"
  side="{{ side }}"
  style="margin-left: auto;"
  class="action_icon diff_scroll_positioned"
  data-testid="diff-"
><a-a style="position:absolute;top:-40px;">
{#- Spetial element to adjust the scroll -#}
</a-a>{% include "_res/svg_ico16_sync.jinja" %}</button>
