{% extends "base.jinja.html" %}
{% set template_type = "Traceability matrix" %}

{% block head_css %}
  {{ super() }}
  <link rel="stylesheet" href="{{ view_object.render_static_url('requirement-tree.css') }}"/>
  <link rel="stylesheet" href="{{ view_object.render_static_url('traceability_matrix_screen.css') }}"/>
{% endblock head_css %}

{% block head_scripts %}
  {{ super() }}
  <script src="{{ view_object.render_static_url('resizable_bar.js') }}"></script>
  <script src="{{ view_object.render_static_url('pan_with_space.js') }}"></script>
  <script src="{{ view_object.render_static_url('traceability_matrix_screen.js') }}"></script>
{% endblock head_scripts %}

{% block title %}
  {{ view_object.project_config.project_title }} - {{ template_type }}
{% endblock title %}

{% block viewtype %}traceability-matrix{% endblock viewtype %}

{% block layout_nav %}
  {% include "_shared/nav.jinja.html" %}
{% endblock layout_nav %}

{% block tree_content %}
  {% include "screens/traceability_matrix/resizable_bar_with_project_tree.jinja" %}
{% endblock tree_content %}

{% block header_content %}
  {%- with header__pagetype=template_type -%}
    {% include "components/header/index.jinja" %}
  {%- endwith -%}
{% endblock header_content %}

{% block main_content %}
  {% include "screens/traceability_matrix/main.jinja" %}
{% endblock main_content %}
