<turbo-stream action="replace" target="article-{{ view_object.document.reserved_mid }}">
  <template>
    {% include "screens/document/document/frame_document_config.jinja.html" %}
  </template>
</turbo-stream>
<turbo-stream action="replace" target="header_document_title">
  <template>
    {%- with standalone=False -%}
      {% include "screens/document/_shared/frame_header_document_title.jinja" %}
    {%- endwith -%}
  </template>
</turbo-stream>
