<turbo-stream action="replace" target="article-{{ document.reserved_mid }}">
  <template>
    {% include "screens/document/document/frame_document_config_edit.jinja.html" %}
  </template>
</turbo-stream>
