{#
  In this template:
  sdoc_entity = requirement || section
  document from _extends_
  node_controls=True
#}

{#
  Example:
  {% with requirement=node, document=node.document, node_controls=True %}
    {%- include "components/node_content/tiny_extends_card.jinja" -%}
  {% endwith %}
#}

<sdoc-node-controls data-direction="row">
  <a
    href="{{ view_object.link_renderer.render_node_link(sdoc_entity, view_object.document, view_object.link_document_type, allow_local=False) }}"
    class="action_button"
    title="Find it in the document view"
    data-testid="{{ sdoc_entity.get_type_string() }}-find-in-document"
  >{% include "_res/svg_ico16_go_to_doc.jinja" %}</a>
  <turbo-frame>
    <a
      href="/actions/show_full_node?reference_mid={{ sdoc_entity.reserved_mid }}"
      class="action_button"
      data-turbo="true"
      data-turbo-action="replace"
      title="Show in full in modal"
      data-testid="node-show-more-action"
    >{% include "_res/svg_ico16_maximize.jinja" %}</a>
  </turbo-frame>
</sdoc-node-controls>
