<turbo-stream action="replace" target="article-{{ form_object.requirement_mid }}">
  <template>
    {% include "screens/document/document/frame_requirement_form.jinja" %}
  </template>
</turbo-stream>
