<div
  class="main"
>
  <div class="main_sticky_header">
    {%- include "components/form/search.jinja" -%}
  </div>

  {%- if view_object.search_results|length > 0 -%}
    <div class="content">
    {% for node in view_object.search_results %}
      {% if node.is_requirement() %}
        {% with requirement=node, document=node.document, node_controls=True %}
          {%- include "components/node_content/tiny_extends_card.jinja" -%}
        {% endwith %}
      {% elif node.is_section() %}
        {% with section=node, document=node.document, node_controls=True %}
          {%- include "components/section/tiny_extends_card.jinja" -%}
        {% endwith %}
      {% elif node.is_source_file() %}
        <sdoc-node node-style="card" node-role="requirement" data-testid="node-source-file">
          <sdoc-node-field>
          <a href="{{ view_object.render_source_file_link_from_root_2(node.source_file) }}">{{ node.source_file.in_doctree_source_file_rel_path }}</a>
          </sdoc-node-field>
          <sdoc-node-field>
          Lines: {{ node.file_stats.lines_total }}
          </sdoc-node-field>
          <sdoc-node-field>
          Connected requirements: {{ node.ng_map_reqs_to_markers.keys() | length }}
          </sdoc-node-field>
          <sdoc-node-field>
          Coverage: {{ node.get_coverage() }}%
          </sdoc-node-field>
        </sdoc-node>
      {% endif %}
    {% endfor %}
    </div>
  {%- else -%}
    {%- include "screens/search/legend.jinja" -%}
  {%- endif -%}
</div> {# /.main #}
