<svg class="svg_icon icon_collapse_expand" width="16px" height="16px" viewBox="0 0 16 16" version="1.1" xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink">
  <polyline class="collapsed" points="13 7 8 12 3 7"></polyline>
  <line class="expanded" x1="4" y1="8" x2="12" y2="8"></line>
</svg>
