{%- if view_object.project_config.is_running_on_server -%}
{#
  This turbo frame is important because it enables
  the child turbo links below.
#}
<turbo-frame>
  <div class="actions_group">
    <a
      class="action_button"
      href="/actions/project_index/new_document"
      data-turbo="true"
      title="Add new document"
      data-testid="tree-add-document-action"
    >{% include "_res/svg_ico16_add.jinja.html" %} Add document</a>

    {%- if view_object.project_config.is_activated_reqif() -%}
      <a
        class="action_button"
        href="/actions/project_index/import_reqif_document_form"
        data-turbo="true"
        title="Import document tree from ReqIF"
        data-testid="tree-import-reqif-action"
      >Import from ReqIF</a>

      {%- if not view_object.is_empty_tree() -%}
        <a
          class="action_button"
          href="/reqif/export_tree"
          title="Export document tree to ReqIF"
          data-testid="tree-export-reqif-action"
        >Export to ReqIF</a>
      {%- endif -%}
    {%- endif -%}
  </div>
</turbo-frame>
{%- else -%}
  {% include "components/static_search/index.jinja" %}
{%- endif -%}
