{% set relation_row_context = namespace() %}
{# The variable 'field' is generated by Python in the main_router.py file. #}
{% set relation_row_context.errors = [] %}
{% set relation_row_context.field = field %}
{% set relation_row_context.form_object = form_object %}
{% set relation_row_context.relation_types = relation_types %}

<turbo-stream action="before" target="requirement_{{ requirement_mid }}__new_relation">
  <template>
    {% include "components/form/row/row_with_relation.jinja" %}
  </template>
</turbo-stream>
