{# needs sdoc_entity, see README.txt #}
{%- if sdoc_entity.has_meta -%}
  {% for meta_field in sdoc_entity.enumerate_meta_fields(skip_multi_lines=True) if view_object.current_view.includes_field(sdoc_entity.node_type, meta_field[0]) %}
    <sdoc-node-field-label>{{ meta_field[0] }}:</sdoc-node-field-label>
    <sdoc-node-field
      data-field-type="singleline"
      data-field-label="{{ meta_field[0] }}"
    >
      {%- with field_content = meta_field[1].get_text_value() %}
        {%- include "components/field/index.jinja" -%}
      {%- endwith -%}
    </sdoc-node-field>
  {% endfor %}
{%- endif %}
