<svg class="svg_icon" width="16" height="16" viewBox="0 0 16 16" version="1.1" xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink">
  <line x1="14.5" y1="14.5" x2="11" y2="11" stroke-linecap="round"></line>
  <circle cx="7" cy="7" r="5"></circle>
</svg>
