<a
  href="{{ delete_href }}"
  class="action_button"
  data-turbo="true"
  data-turbo-method="delete"
  data-action-type="delete"
  data-testid="form-delete-action"
>{{ delete_name|default('Delete') }}</a>
