{# needs sdoc_entity #}

{% set node_type_string = sdoc_entity.get_node_type_string() %}

  <sdoc-node
    node-style="readonly"
    node-role="{{ sdoc_entity.get_type_string() }}"
    {%- if node_type_string is not none %}
      show-node-type-name="{{ node_type_string }}"
    {%- endif %}
    {% if sdoc_entity.is_requirement %}
      node-view="{{ sdoc_entity.get_requirement_style_mode() }}"
    {%- endif -%}
    data-testid="node-{{ sdoc_entity.get_type_string() }}"
  >

    {% include "components/anchor/index.jinja" %}
    {# sdoc_entity #}
    {% block sdoc_entity %}
    {# {% include "components/"~sdoc_entity.get_type_string()~"/card.jinja" %} #}
    {% endblock sdoc_entity %}

  </sdoc-node>
