{% extends "components/node/card.jinja" %}
{%- set sdoc_entity = requirement -%}
{%- set requirement_style = "simple" -%}
{% block sdoc_entity %}
{% include "components/node_content/tiny.jinja" %}
{% endblock sdoc_entity %}
