{#
  In this template:
  sdoc_entity = section
#}

<sdoc-section>
  {% include "components/node_field/section_h/index.jinja" %}
  {% include "components/node_field/uid_standalone/index.jinja" %}
</sdoc-section>
