{# All variables for this template are generated by Python in the main_router.py file. #}

<turbo-stream action="{{ replace_action }}" target="article-{{ target_node_mid }}">
  <template>
    {% include "screens/document/document/frame_requirement_form.jinja" %}
  </template>
</turbo-stream>
