{% extends "base.jinja.html" %}
{% set template_type = "DIFF" %}

{% block head_css %}
  {{ super() }}
  <link rel="stylesheet" href="{{ view_object.render_static_url('diff.css') }}"/>
{% endblock head_css %}

{% block head_scripts %}
  {{ super() }}

  <script src="{{ view_object.render_static_url('stimulus_umd.min.js') }}"></script>
  <script>
    Stimulus.application = Stimulus.Application.start();
  </script>

  {%- if view_object.project_config.is_running_on_server and not view_object.standalone -%}
  <script type="module">
    import hotwiredTurbo from "{{ view_object.render_static_url_with_prefix('turbo.min.js') }}";
  </script>
  <script src="{{ view_object.render_static_url('controllers/modal_controller.js') }}"></script>
  {%- endif -%}
  <script src="{{ view_object.render_static_url('diff.js') }}"></script>
{% endblock head_scripts %}

{% block title %}
  {{ view_object.project_config.project_title }} - {{ template_type }}
{% endblock title %}

{% assert view_object.tab in ("diff", "changelog") %}
{% block viewtype %}{{ view_object.tab }}{% endblock viewtype %}

{% block layout_nav %}
  {% include "_shared/nav.jinja.html" %}
{% endblock layout_nav %}

{% block tree_content %}
   {# NOTHING #}
{% endblock tree_content %}

{% block header_content %}
  {%- with header__pagetype=template_type -%}
    {% include "components/header/index.jinja" %}
  {%- endwith -%}
{% endblock header_content %}

{% block main_content %}
  {% include "screens/git/main.jinja" %}
{% endblock main_content %}
