{% include "_res/svg_ico16_code.jinja.html" %}
<div class="header__document_title"
  title="{{ view_object.source_file.in_doctree_source_file_rel_path }}">
  {{ view_object.source_file.in_doctree_source_file_rel_path }}
</div>
