{%- if view_object.document.has_any_requirements() -%}
  <div
    class="main"
    js-pan_with_space="true"
  >
    {% include "_shared/tags.jinja.html" %}
    <div class="content">
      {%- for section_or_requirement, _ in view_object.document_content_iterator() %}
        {%- if section_or_requirement.is_section() and section_or_requirement.ng_has_requirements %}
          <section class="content_section">
            <div class="content_item" data-role="current">
              {%- with section = section_or_requirement, node_controls = True -%}
                {%- include "components/section/tiny_extends_card.jinja" -%}
              {%- endwith -%}
            </div>
          </section>
        {%- elif section_or_requirement.is_requirement() and not section_or_requirement.is_text_node() %}
          <section class="content_section">
            <div class="content_item" data-role="parents">
              {%- with requirement=section_or_requirement, node_controls = True, requirement_partial="components/node_content/tiny_extends_card.jinja" -%}
                {% include "_shared/requirement_tree_left.jinja.html" %}
              {%- endwith -%}
            </div>
            <div class="content_item" data-role="current">
              {%- with requirement = section_or_requirement, node_controls = True -%}
                {%- include "components/node_content/tiny_extends_card.jinja" -%}
              {%- endwith -%}
            </div>
            <div class="content_item" data-role="children">
              {%- with requirement=section_or_requirement, node_controls = True, requirement_partial="components/node_content/tiny_extends_card.jinja" -%}
                {%- include "_shared/requirement_tree_right.jinja.html" -%}
              {%- endwith -%}
            </div>
          </section>
        {%- endif -%}
      {%- endfor -%}
    </div> {# /.content #}
  </div> {# /.main #}
{%- else -%}
  <sdoc-main-placeholder data-testid="document-main-placeholder">
    This view is empty because
    <br/>
    the document has no requirements.
  </sdoc-main-placeholder>
{%- endif -%}
