<div class="source-file__aside">
  <sdoc-tabs class="in_aside_panel">
    <sdoc-tab data-testid="source-file-tab-Nodes" active="" style="order: 0;">Nodes</sdoc-tab>
    <sdoc-tab data-testid="source-file-tab-Ranges" style="order: 1;">Ranges</sdoc-tab>
  </sdoc-tabs>

  <div class="source-file__refer" id="referContainer">
  <sdoc-tab-content id="Nodes" active>
    <div class="source-file__toc">
      {%- set source_file_range_reqs =
        view_object.traceability_index.get_source_file_reqs(view_object.source_file.in_doctree_source_file_rel_path_posix) -%}

      {%- if source_file_range_reqs -%}
        {%- for requirement in source_file_range_reqs -%}
          <div class="source-file__toc-node">
            {{ view_object.render_aside_requirement(requirement.reserved_uid) }}
          </div>
        {%- endfor -%}
      {%- endif -%}
    </div>
  </sdoc-tab-content>
  <sdoc-tab-content id="Ranges">
    <div class="source-file__toc">
      {%- for line in view_object.pygmented_source_file_lines -%}
        {%- if line.__class__.__name__ == "SourceMarkerTuple" and not line.is_end() -%}
          <div class="source-file__toc-range">
            <div class="source-file__toc-range-header">
              {%- with
                begin=line.ng_range_line_begin,
                end=line.ng_range_line_end,
                href=view_object.render_marker_range_link(line.markers[0]),
                scope=line.markers[0].get_description()
              -%}
                {% include "screens/source_file_view/range_button.jinja" %}
              {%- endwith -%}
            </div>
            {% for marker_ in line.markers %}
              {% for node_uid_ in marker_.reqs %}
                <div class="source-file__toc-range-node">
                {{ view_object.render_aside_requirement(node_uid_, marker_.ng_range_line_begin, marker_.ng_range_line_end) }}
                </div>
              {% endfor %}
            {% endfor %}
          </div>
        {%- endif -%}
      {%- endfor -%}
    </div>
  </sdoc-tab-content>
  </div>

</div>
