<div class="tree">
  {%- for document in view_object.iterate_documents() -%}
    <a
      href="#{{ loop.index }}"
      class="tree_item"
      data-testid="tree-document-anchor"
    >
      {% include "_res/svg_ico16_document.jinja.html" %}
      <div
        class="document_title"
        title="{{ document.title }}"
        data-file_name=""
      >{{ document.title }}</div>
    </a>
  {%- endfor -%}
</div>
