{#
  Requires: copy_stable_link_button_controller.js in index.jinja
#}
{%- if path is defined -%}
  <div
    data-controller="copy_stable_link_button"
    data-path="{{ path }}"
    class="copy_stable_link-button"
    title="Click to copy a stable node link to the clipboard"
  >
    <span style="display: contents;" class="copy_to_clipboard-copy_icon">
      {% include "_res/svg_ico16_link.jinja" %}
    </span>
    <span style="display: none;" class="copy_to_clipboard-done_icon">
      {% include "_res/svg_ico16_done.jinja" %}
    </span>
  </div>
{%- endif -%}
