{# needs resizable_bar.js #}
{#
  This template expects content URLs for the bar,
  passed as variable `resizable_bar_content`:
    {%- with
      resizable_bar_content = 'path/to/file/file.jinja',
      resizable_bar_name = 'name'
    -%}
    {% include "components/resizable_bar/index.jinja" %}
    {%- endwith -%}
#}
{#
  Can be optionally defined as well:
      resizable_bar_state = 'open' | 'closed',
      resizable_bar_position = 'left' | 'right',
#}

<div
  js-resizable_bar="{{ resizable_bar_name }}"
  data-state="{{ resizable_bar_state|default('open', true) }}"
  data-position="{{ resizable_bar_position|default('left', true) }}"
>
{% include resizable_bar_content %}
</div>
