{% set section_change = view_object.change_stats.find_change(section) %}

<details
  class="diff_node"
  {% if section_change is not none %}
    modified="{{ side }}"
  {% endif %}
>
  <summary>
    {%- with badge_text = "S" -%}
      {%- include "components/badge/index.jinja" -%}
    {%- endwith -%}
    <span>
      {{ section.context.title_number_string if section.context.title_number_string else "&nbsp;"|safe * (section.ng_level * 2 - 1) }}
    </span>
    <span>{{ section.reserved_title }}</span>
    {%- if tab == "diff" -%}
      {%- if section_change is not none -%}
        {%- if section_change.section_token is not none -%}
          {%- with uid=section_change.section_token -%}
            {%- include "screens/git/sync/button.jinja" -%}
          {%- endwith -%}
        {%- endif -%}
      {%- endif -%}
    {%- endif -%}
  </summary>

  {% include "screens/git/fields/section_fields.jinja" %}
</details>
