<turbo-frame id="header_document_title">
  {% include "_res/svg_ico16_document.jinja.html" %}
  <div
    class="header__document_title"
    title="{{ view_object.document.title }}"
  >
    {{ view_object.document.title }}
  </div>
</turbo-frame>
