<turbo-frame id="frame_document_content">
      <div
        class="content"
      >
        {# FIXME: Task #1229 / Uncomment when validation logic is implemented: #}
        {# {% include "screens/document/document/frame_document_issues.jinja" %} #}

        {% include "screens/document/document/frame_document_config.jinja.html" %}

        {%- for node, _ in view_object.document_content_iterator() %}

        {%- if node.is_section() or node.is_document() %}
          {% include "components/section/index_extends_node.jinja" %}
        {%- elif node.is_requirement() %}
          {# {%- if node.is_text_node() %}
            {% include "components/text_node/index_extends_node.jinja" %}
          {% else %} #}
            {% include "components/node_content/index_extends_node.jinja" %}
          {# {% endif %} #}
        {%- else %}
          {% assert false, "Must not reach here." %}
        {%- endif %}

        {%- endfor %}

      </div>
</turbo-frame>
