{#
{%- with
  begin=current_range_begin,
  end=current_range_end,
  href=current_marker_link,
  scope=line__.marker__.get_description()
-%}
  {% include "screens/source_file_view/range_button.jinja" %}
{%- endwith -%}
#}

<a
  class="source__range-pointer action_button"
  data-begin="{{ begin }}"
  data-end="{{ end }}"
  href="{{ href }}"
  title="Click to toggle range (Ctrl/Cmd + Click with focus)"
><span class="source__range-definition">{{ begin }} - {{ end }}{% if scope is defined %} | {{ scope }}{% endif %}</span></a>
