{# needs sdoc_entity, see README.txt #}
{%- if sdoc_entity.has_reserved_statement() and view_object.current_view.includes_field(sdoc_entity.node_type, sdoc_entity.get_content_field_name()) -%}
  {%- if truncated_statement is true -%}
    {# truncated in DTR tiny card #}
    <sdoc-node-field data-field-label="statement">
      <sdoc-autogen>{{ view_object.render_truncated_node_statement(sdoc_entity) }}</sdoc-autogen>
    </sdoc-node-field>
  {%- else -%}
    {# default with label #}
    <sdoc-node-field-label>{{ sdoc_entity.get_field_human_title_for_statement() }}:</sdoc-node-field-label>
    <sdoc-node-field
      data-field-label="statement"
    >
      {%- with field_content = view_object.render_node_statement(sdoc_entity) %}
        {%- include "components/field/index.jinja" -%}
      {%- endwith -%}
    </sdoc-node-field>
  {%- endif -%}
{%- endif -%}
