{#
  In this template:
  sdoc_entity = requirement
#}

{# TODO: card display #}
{# Now in the cards we show all fields as in the index #}
{%- set requirement_style = "simple" -%}
{%- set sdoc_entity = requirement -%}
{% include "components/node_content/index.jinja" %}
