<div class="nav">

{# TODO: title="Project name" #}

  <a
    data-link="index"
    class="nav_button"
    href="{{ view_object.render_url('index.html') }}"
    title="Project index">
    {% include "_res/svg_ico16_index.jinja.html" %}
  </a>

{%- if view_object.project_config.is_activated_project_statistics() -%}
  <a
    data-link="project_information"
    class="nav_button"
    href="{{ view_object.render_url('project_statistics.html') }}"
    title="Project statistics"
    data-testid="project-tree-link-project-statistics"
  >
    {% include "_res/svg_ico16_stat.jinja" %}
  </a>
{%- endif -%}

{%- if view_object.project_config.is_activated_requirements_coverage() -%}
  <a
    data-link="traceability-matrix"
    class="nav_button"
    href="{{ view_object.render_url('traceability_matrix.html') }}"
    title="Traceability matrix"
    data-testid="project-tree-link-requirements-coverage"
  >
    {% include "_res/svg_ico16_requirement.jinja.html" %}
  </a>
{%- endif -%}

{%- if view_object.project_config.is_activated_requirements_to_source_traceability() -%}
  <a
    data-link="source_coverage"
    class="nav_button"
    href="{{ view_object.render_url('source_coverage.html') }}"
    title="Source coverage"
    data-testid="project-tree-link-source-coverage"
  >
    {% include "_res/svg_ico16_source.jinja.html" %}
  </a>
{%- endif -%}

{%- if view_object.project_config.is_activated_search() -%}
  <a
    data-link="search"
    class="nav_button"
    href="{{ view_object.render_url('search') }}"
    title="Search"
    data-testid="project-tree-link-search"
  >
    {% include "_res/svg_ico16_search.jinja" %}
  </a>
{%- endif -%}

{%- if view_object.project_config.is_running_on_server and view_object.project_config.is_activated_diff() -%}
  <a
    data-link="diff"
    class="nav_button"
    href="{{ view_object.render_url('diff') }}"
    title="Diff"
    data-testid="project-tree-link-diff"
  >
    {% include "_res/svg_ico16_diff.jinja" %}
  </a>
{%- endif -%}

</div>
