{% set template_type = 'Source file coverage' %}

{% extends "base.jinja.html" %}

{% block head_css %}
  {{ super() }}
  <link rel="stylesheet" href="{{ view_object.render_static_url('source_file_screen.css') }}" />
  <link rel="stylesheet" href="{{ view_object.render_static_url('requirement-tree.css') }}"/>
{% endblock head_css %}
{% block head_scripts %}
  <script src="{{ view_object.render_static_url('source_file_screen.js') }}"></script>
  {{ super() }}
{% endblock head_scripts %}

{% block title %}
  {{ view_object.source_file.in_doctree_source_file_rel_path_posix }} - {{ template_type }}
{% endblock title %}

{% block viewtype %}source-file{% endblock viewtype %}

{% block head %}
  {{ super() }}

  <style>
    {{ view_object.pygments_styles }}
  </style>
{% endblock head %}

{% block layout_nav %}
  {% include "_shared/nav.jinja.html" %}
{% endblock layout_nav %}

{% block header_content %}
  {%- with
    header__items=["screens/source_file_view/header__source_file_name.jinja"],
    header__pagetype=template_type,
    header__last="screens/source_file_view/header__actions.jinja"
  -%}
    {% include "components/header/index.jinja" %}
  {%- endwith -%}
{% endblock header_content %}

{% block tree_content %}
  {% include "screens/source_file_view/aside.jinja" %}
{% endblock tree_content %}

{% block main_content %}
  {% include "screens/source_file_view/main.jinja" %}
{% endblock main_content %}
