{% if view_object.should_display_included_documents_for_document(document) %}
<ul class="tree_fragments">
{% for child_document_ in document.included_documents %}
  <li>
    <a
      class="tree_item"
      {% if view_object.document == child_document_ %}
      active="true"
      {% endif %}
      data-testid="tree-document-fragment-link"
      href="{{ view_object.document.meta.get_root_path_prefix() }}/{{ child_document_.meta.get_html_doc_link() }}"
    >
      {# TODO #fragment #}
      {%- include "_res/svg_ico16_fragment_draft.jinja" -%}
      {# {% include "_res/svg_ico16_fragment.jinja" %} #}
      <div
        class="document_title"
        title="{{ child_document_.title }}"
        data-file_name="{{ child_document_.meta.document_filename }}"
      >{{ child_document_.title }}</div>
    </a>

    {% with document=child_document_ %}
    {% include "screens/document/_shared/project_tree_child_documents.jinja" %}
    {% endwith %}
  </li>
{% endfor %}
</ul>
{% endif %}
