<turbo-stream action="replace" target="frame_document_content">
  <template>
    {% include "screens/document/document/frame_document_content.jinja.html" %}
  </template>
</turbo-stream>
<turbo-stream action="update" target="confirm">
  <template></template>
</turbo-stream>
