{# needs sdoc_entity, see README.txt #}
{%- if sdoc_entity.reserved_uid %}
  <sdoc-node-uid>
    {%- with field_content = sdoc_entity.reserved_uid %}
      {%- include "components/field/index.jinja" -%}
    {%- endwith -%}
  </sdoc-node-uid>
{%- endif -%}
