{% extends "base.jinja.html" %}

{% block head_css %}
  {{ super() }}
  <link rel="stylesheet" href="{{ view_object.render_static_url('table_vew.css') }}"/>
{% endblock head_css %}

{% block head_scripts %}
  <script src="{{ view_object.render_static_url('viewtype_menu.js') }}"></script>
  <script src="{{ view_object.render_static_url('resizable_bar.js') }}"></script>
  <script src="{{ view_object.render_static_url('collapsible_toc.js') }}"></script>
  <script src="{{ view_object.render_static_url('toc_highlighting.js') }}"></script>

  {%- if view_object.project_config.is_activated_mathjax() -%}
  <script id="MathJax-script" async src="{{ view_object.render_static_url('mathjax/tex-mml-chtml.js') }}"></script>
  {%- endif -%}
  {%- if view_object.project_config.is_activated_rapidoc() -%}
  <script src="{{ view_object.render_static_url('rapidoc/rapidoc-min.js') }}"></script>
  {%- endif -%}
  {%- if view_object.project_config.is_activated_mermaid() -%}
    <script src="{{ view_object.render_static_url('mermaid/mermaid.min.js') }}"></script>
    <script type="module">
      mermaid.initialize({ startOnLoad: true });
    </script>
  {%- endif -%}
  {{ super() }}
{% endblock head_scripts %}
{% block title %}{{ view_object.document.title }} - {{ view_object.document_type.get_page_title() }}{% endblock title %}
{% block viewtype %}table{% endblock viewtype %}

{% block layout_nav %}
  {%- if not view_object.standalone -%}
  {% include "_shared/nav.jinja.html" %}
  {%- endif -%}
{% endblock layout_nav %}

{% block tree_content %}
  {% include "screens/document/_shared/resizable_bar_with_project_tree.jinja" %}
{% endblock tree_content %}

{% block toc_content %}
  {% include "screens/document/_shared/resizable_bar_with_toc.jinja" %}
{% endblock toc_content %}

{% block header_content %}
  {%- with
      header__items=[
        "screens/document/_shared/frame_header_document_title.jinja",
        "screens/document/_shared/viewtype_menu.jinja"
      ]
  -%}
    {% include "components/header/index.jinja" %}
  {%- endwith -%}
{% endblock header_content %}

{% block main_content %}
  {% include "screens/document/table/main.jinja" %}
{% endblock main_content %}
