<turbo-stream action="update" target="confirm">
  <template>
  {% with
    confirm_title = default,
    confirm_message = "Deleting a requirement is an unrecoverable action.",
    confirm_name = "Delete requirement",
    confirm_href = "/actions/document/delete_requirement?node_id="~requirement_mid~"&context_document_mid="~context_document_mid~"&confirmed=1"
  %}
    {% include "components/confirm/index.jinja" %}
  {% endwith %}
  </template>
</turbo-stream>
