<div class="diff_node_fields">
  {# MID field, if permanent, is always printed as-is. It is never marked as modified. #}
  {% if section.mid_permanent %}
    <div
      class="diff_node_field"
    >
      {%- with badge_text = "MID" -%}
        {%- include "components/badge/index.jinja" -%}
      {%- endwith -%}
      <span class="sdoc_pre_content">{{ section.reserved_mid }}</span>
    </div>
  {% endif %}

  {# UID #}
  {%- if section.reserved_uid is not none and section.reserved_uid|length > 0 -%}
  {% set uid_modified = section_change is not none and section_change.uid_modified %}
    <div
      class="diff_node_field"
      {% if uid_modified %}
        modified="{{ side }}"
      {% endif %}
    >
      {%- with badge_text = "UID" -%}
        {%- include "components/badge/index.jinja" -%}
      {%- endwith -%}
      <div class="sdoc_pre_content">{{ section.reserved_uid }}</div>
    </div>
  {%- endif %}

  {# TITLE #}
  {% set title_modified = section_change is not none and section_change.title_modified %}
  <div
    class="diff_node_field"
    {% if title_modified %}
      modified="{{ side }}"
    {% endif %}
  >
    {%- with badge_text = "TITLE" -%}
      {%- include "components/badge/index.jinja" -%}
    {%- endwith -%}
    {% if title_modified %}
      {%- if section_change.is_paired_change() -%}
        <div class="sdoc_pre_content">{{ section_change.get_colored_title_diff(side) }}</div>
      {%- else -%}
        <div class="sdoc_pre_content">{{ section.reserved_title }}</div>
      {%- endif -%}
    {% else %}
    <div class="sdoc_pre_content">{{ section.reserved_title }}</div>
    {% endif %}
  </div>

</div>
